import numpy as np

def sliding_window_view(arr, shape):
    """
    This function was written by fnjn on github and posted to numpy/numpy.
    It takes in an array and a shape, and then gives back an array that is
    essentially moving around a planchette of the described shape over the
    array.
    """
    n = arr.shape
    o = n - shape + 1 # output shape
    strides = arr.strides
    new_shape = np.concatenate((o, shape), axis=0)
    new_strides = np.concatenate((strides, strides), axis=0)
    return np.lib.stride_tricks.as_strided(arr ,new_shape, new_strides)

def fnjn_mvgavg(a, n, axis=0, weights=None):
    shape = a.shape-(n-1)*np.eye(a.ndim,dtype='int')[axis]
    window = sliding_window_view(a, shape)
    if weights:
        window = window.swapaxes(0,axis)
        assert len(window)==len(weights)
        window=np.array([window[n]*weights[n] for n in range(len(weights))])/np.sum(weights)
        window = window.swapaxes(0,axis)
    return np.mean(window,axis=axis).reshape(shape)


def cumsum_mvgavg(a, n, axis=0):
    table = np.cumsum(a.swapaxes(0,axis),axis=0)/n
    if table.ndim > 1:
        table = np.vstack([[0*table[0]],table])
    else:
        table = np.array([0,*table])
    return np.swapaxes(table[n:]-table[:-n],0,axis)


def binning_mvgavg(a,n,axis=0):
    a=a.swapaxes(0,axis)
    return a[:int(len(a)/n)*n].reshape(int(len(a)/n),n,*a.shape[1:]).mean(axis=1).swapaxes(0,axis)    



from setuptools import setup, find_packages

exec(open('mvgavg/version.py').read()) # loads __version__

setup(name='mvgavg',
      version=__version__,
      author='NGeorgescu',
    description='Moving averages',
    long_description=open('README.rst').read(),
    license='see LICENSE.txt',
    keywords="mvgavg moving average movingaverage numpy python binning binned running run",
    install_requires=['numpy'],
    packages= find_packages())






import numpy as np

from .mvgavg import fnjn_mvgavg, cumsum_mvgavg, binning_mvgavg

def mvgavg(a, n, axis=0, weights=False, binning=False):
    """
    Performs a moving average along a single axis

    Parameters
    ----------
    a : array_like
        Array to be averaged
    n : int
        Width of the moving average
    axis: int
        Axis along which the moving average is to take place, 0 if unspecified
    weights: 'pascal', 'triangle', 'quadratic', or a list
        weights to multiply the moving average by
    binning:
        Bins the values within the moving window together and moves on to the next

    Returns
    -------
    mvgavg : nd_array
        An array with the elements averaged over a length of `n`.  The array will be
        of a length `n-1` shorter than the original along the chosen axis, unless 
        `binning=True`, in which case it will return an array `n` times shorter along
        the chosen axis.
    
    See Also
    --------
    ndarray.average : equivalent to setting n equal to the length of data along
        the specified axis
    pandas.DataFrame.rolling : provides a rolling window calculation in pandas
    

    Notes
    -----
    Consider the list `[a, b, c, d, e....]` with a moving average of length 3.  The
    First element in the list will be `(a+b+c)/3`.  The next `(b+c+d)/3`.  And so on.
    The result is a smoothed version of the original data with length (n-1) shorther
    than the original data.  This works with arrays of any dimension, summing along
    as it goes. This function will sometimes fail for a ragged array.  Use a list
    comprehension with np.mvgavg embedded in it instead.

    The axis parameter controls the axis over which the moving average happens. In a
    2D array, for instance, for `axis=0`, the array will average along the columns, and
    axis=1 will average along the rows.

    Consider the array `[a b c d e... ]` that is being averaged to the product array,
    `[A B C D... ]` over a distance of 3. If `binning = False`: ::

        A = [a b c]/3
        B =   [b c d]/3
        C =     [c d e]/3 
        D =       [d e f]/3
        ....

    but if `binning = True`: ::

        A = [a b c]/3
        B =       [d e f]/3
        C =             [g h i]/3
        .....

    Thus, a binned moving average decreases the array length by a factor of `n`. This
    throws out some precision but greatly decreases computational time for very 
    long arrays.

    Examples
    --------
    >>> from mvgavg import mvgavg
    >>> x = [[i,(-1)**i] for i in np.arange(5)]
    >>> mvgavg(x,2)
    array([[0.5, 0. ],
           [1.5, 0. ],
           [2.5, 0. ],
           [3.5, 0. ]])
    >>> mvgavg(x,2,axis=1)
    array([[0.5],
           [0. ],
           [1.5],
           [1. ],
           [2.5]])

    """
    if binning:
        return binning_mvgavg(np.asarray(a),n,axis=axis)
    else:
        if weights:
            return fnjn_mvgavg(np.asarray(a),n,axis=axis,weights=weights)
        else:
            return cumsum_mvgavg(np.asarray(a),n,axis=axis)



